// Service.java - provide services to Gluon Cloud
// Copyright 2015, Anderson Software Group, Inc.
// Gail and Paul Anderson

package com.asgteach.bpcloud.service;

import com.asgteach.bpcloud.model.BPData;
import com.gluonhq.connect.ConnectState;
import com.gluonhq.connect.GluonObservableList;
import com.gluonhq.connect.gluoncloud.GluonClient;
import com.gluonhq.connect.gluoncloud.SyncFlag;
import com.gluonhq.connect.gluoncloud.User;
import com.gluonhq.connect.provider.DataProvider;
import javafx.beans.Observable;
import javafx.beans.property.ListProperty;
import javafx.beans.property.ObjectProperty;
import javafx.beans.property.SimpleListProperty;
import javafx.beans.property.SimpleObjectProperty;
import javafx.collections.FXCollections;
import javafx.util.Callback;
import javax.annotation.PostConstruct;

/**
 *
 * @author gail
 */
public class Service {

    private final ObjectProperty<User> user = new SimpleObjectProperty<>();

    // include property extractors so we can listen for updates
    private final ListProperty<BPData> bpList
            = new SimpleListProperty<>(FXCollections.observableArrayList((BPData param) -> new Observable[]{
                param.systolicProperty(),
                param.diastolicProperty(),
                param.pulseProperty()
            }));
    

    private GluonClient gluonClient;

    @PostConstruct
    public void postConstruct() {
        user.addListener((obs, ov, nv) -> {
            if (nv != null) {
                System.out.println(nv.getKey()
                    + " for "
                    + nv.getName());
            }
        });
        gluonClient = GluonClientProvider.getGluonClient();
        System.out.println("got gluon client");
        user.bind(gluonClient.authenticatedUserProperty()); 
    }

    // Call this after authentication
    // Store user's data in the cloud based on the user "nickname" and
    // "networkID". It should be unique for each user.
    // Requires that the user authenticates via the same social network site
    // to access the data from other devices.
    private void getData() {
        System.out.println("Get Data called.");
        
        GluonObservableList<BPData> gluonBPData = DataProvider.<BPData>retrieveList(
                gluonClient.createListDataReader(
                        user.get().getNick() + user.get().getNetworkId(), 
                        BPData.class,
                        SyncFlag.LIST_READ_THROUGH, SyncFlag.LIST_WRITE_THROUGH,
                        SyncFlag.OBJECT_READ_THROUGH, SyncFlag.OBJECT_WRITE_THROUGH));

        gluonBPData.stateProperty().addListener((obs, ov, nv) -> {
            if (ConnectState.SUCCEEDED.equals(nv)) {
                System.out.println("Service-getData with authenticated user : connection succeeded.");
                bpList.set(gluonBPData);
            }
        });
    }

    public void retrieveReadings() {
        System.out.println("retrieveReadings");
        gluonClient.authenticate(this::getData);
    }

    public BPData addReading(BPData data) {
        bpList.get().add(data);
        return data;
    }

    public void removeReading(BPData data) {
        bpList.get().remove(data);
    }

    public ListProperty<BPData> bpListProperty() {
        return bpList;
    }

    public void clear() {
        System.out.println("clearing " + bpList.get().size() + " items.");
        bpList.get().clear();
    }
    
    public ObjectProperty<User> userProperty() {
        return user;
    }
    
    public User getUser() {
        return user.get();
    }

}
