/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package com.asgteach.bpmonitor.views;

import com.asgteach.bpmonitor.BPMonitor;
import static com.asgteach.bpmonitor.BPMonitor.HOME_VIEW;
import static com.asgteach.bpmonitor.BPMonitor.GRAPH_VIEW;
import com.asgteach.bpmonitor.model.BPData;
import com.asgteach.bpmonitor.model.Model;
import com.asgteach.bpmonitor.service.Service;
import com.gluonhq.charm.glisten.animation.BounceInRightTransition;
import com.gluonhq.charm.glisten.application.MobileApplication;
import com.gluonhq.charm.glisten.control.AppBar;
import com.gluonhq.charm.glisten.control.Avatar;
import com.gluonhq.charm.glisten.mvc.View;
import com.gluonhq.charm.glisten.visual.MaterialDesignIcon;
import java.time.format.DateTimeFormatter;
import javafx.beans.binding.Bindings;
import javafx.collections.transformation.FilteredList;
import javafx.collections.transformation.SortedList;
import javafx.fxml.FXML;
import javafx.scene.Node;
import javafx.scene.chart.BarChart;
import javafx.scene.chart.XYChart;
import javafx.scene.control.Button;
import javafx.scene.control.Label;
import javafx.scene.control.Tooltip;
import javafx.scene.image.Image;
import javafx.scene.input.MouseEvent;
import javafx.scene.input.ScrollEvent;
import javafx.scene.input.ZoomEvent;
import javafx.scene.layout.VBox;
import javax.inject.Inject;

/**
 *
 * @author gail
 */
public class StatPresenter {

    @FXML
    private View statView;

    private Button statButton;
    private Button homeButton;
    private Button chartButton;
    private final int MAX = 0;
    private final int AVG = 1;
    private final int MIN = 2;

    // Bar Chart Series
    private final XYChart.Series<String, Number> systolicStats
            = new XYChart.Series<>();
    private final XYChart.Series<String, Number> diastolicStats
            = new XYChart.Series<>();
    private final XYChart.Series<String, Number> pulseStats
            = new XYChart.Series<>();
    @FXML
    private BarChart<String, Number> statChart;
    @FXML
    private Label nodeDisplay;
    @FXML
    private VBox container;
    
    private FilteredList<BPData> filteredList;
    private SortedList<BPData> sortedList;
    private final DateTimeFormatter dateformatter = DateTimeFormatter.ofPattern("MMM dd yyyy");
    
    @Inject
    private Service service;
    @Inject
    private Model model;
    
    private int numReadings = Model.MAXDISPLAY;
    private int maxReadings;
    @FXML
    private Label titleLabel;
    

    public void initialize() {
        statView.setShowTransitionFactory(BounceInRightTransition::new);

        statView.showingProperty().addListener((obs, oldValue, newValue) -> {
            if (newValue) {
                AppBar appBar = MobileApplication.getInstance().getAppBar();
                appBar.setNavIcon(new Avatar(17,
                        new Image(BPMonitor.class.getResourceAsStream("/icon.png"))));
                appBar.setTitleText("Stats");
                
                appBar.getActionItems().add(homeButton = MaterialDesignIcon.HOME.button(e
                        -> MobileApplication.getInstance().switchView(HOME_VIEW)));
                appBar.getActionItems().add(chartButton = MaterialDesignIcon.TIMELINE.button(e
                        -> MobileApplication.getInstance().switchView(GRAPH_VIEW)));
                appBar.getActionItems().add(statButton = MaterialDesignIcon.VIEW_WEEK.button(e
                        -> System.out.println("Stat button")));
                
                homeButton.setTooltip(new Tooltip("VIEW DATA"));
                chartButton.setTooltip(new Tooltip("VIEW LINE CHART"));
                statButton.setTooltip(new Tooltip("VIEW STATS"));
                statButton.setDisable(true);
                displayStatChart();
            }
        });
        
       
        model.getStartIndexProperty().addListener(observable -> {
            System.out.println("start index change");
            filteredList.setPredicate(data -> inRange(sortedList.indexOf(data)));
        });
        
        nodeDisplay.addEventHandler(MouseEvent.MOUSE_CLICKED, event -> {
                nodeDisplay.setText("");
                event.consume();
            });

        // Set up Stat Chart. 
        systolicStats.setName("Systolic");
        statChart.getData().add(systolicStats);
        diastolicStats.setName("Diastolic");
        statChart.getData().add(diastolicStats);
        pulseStats.setName("Pulse");
        statChart.getData().add(pulseStats);

        // Initialize Bar Chart data
        initStatChart(systolicStats);
        initStatChart(diastolicStats);
        initStatChart(pulseStats);


        // process scroll events for vertical scroll only
        container.setOnScroll((ScrollEvent event) -> {
            if (!event.isInertia()) {
                container.setTranslateY(container.getTranslateY() + event.getDeltaY());
            }
            event.consume();
        });

        container.setOnScrollStarted((ScrollEvent event) -> {
            System.out.println("Container: Scroll started event");
            event.consume();
        });

        container.setOnScrollFinished((ScrollEvent event) -> {
            System.out.println("Container: Scroll finished event");
            event.consume();
        });

    }

    private void displayStatChart() {
        
        System.out.println("Service List Property listener: list size = " + service.bpListProperty().get().size());
        filteredList = new FilteredList(service.bpListProperty().get());
        sortedList = new SortedList(filteredList);
        sortedList.setComparator((d1, d2) -> d2.getDate().compareTo(d1.getDate()));
        maxReadings = sortedList.size();
        numReadings = (maxReadings < Model.MAXDISPLAY ? maxReadings : Model.MAXDISPLAY);
        filteredList.setPredicate(data -> inRange(sortedList.indexOf(data)));
        
        nodeDisplay.setText("");
        statChart.setTitle(
                "Blood Pressure Stats for "
                + filteredList.size() +
                " Readings");
        // compute data based on sortedList
        // Unfortunately, can't use streams
        computeMax();
        computeAvg();
        computeMin();
        
        titleLabel.textProperty().bind(Bindings.createStringBinding(() -> {
            if (sortedList.size() <= 0) {
                return "";
            }
            return sortedList.get(0).getDate().format(dateformatter) + " -> "
                    + sortedList.get(sortedList.size() - 1).getDate().format(dateformatter);

        }, model.getStartIndexProperty()));
    }
    
    
    // This is the FilteredList predicate--all based on the startDate control.
    // We convert LocalDateTime to LocalDate--want to ignore time here
    private boolean inRange(int index) {
        return (index >= model.getStartIndex() && index < model.getStartIndex() + numReadings);
    }

    // Initialize Bar Chart with data and then configure it.
    // Use the same data when the Line Chart is empty
    private void initStatChart(XYChart.Series<String, Number> target) {
        target.getData().add(new XYChart.Data<>("Max", 1));
        target.getData().add(new XYChart.Data<>("Avg", 1.0));
        target.getData().add(new XYChart.Data<>("Min", 1));
        System.out.println("iniStatChart for " + target.getName());
        configureBarChartNodes(target);
    }

    // Add MOUSE_CLICKED event handler for the Bar Chart.
    // This displays the YValue data in a Label.
    private void configureBarChartNodes(
            XYChart.Series<String, Number> target) {
        for (XYChart.Data<String, Number> data : target.getData()) {
            Node node = data.getNode();
            node.addEventHandler(MouseEvent.MOUSE_CLICKED, event -> {
                if (data.getXValue().equalsIgnoreCase("Avg")) {
                    nodeDisplay.setText(
                            String.format("%5.1f", data.getYValue().floatValue()));
                } else {
                    nodeDisplay.setText(
                            String.format("%s", data.getYValue()));
                }
                event.consume();
            });
            
        }
    }

    private void computeMax() {

        Integer numSys = 1;
        Integer numDias = 1;
        Integer numPulse = 1;
        for (BPData data : sortedList) {
            if (data.getSystolic() > numSys) {
                numSys = data.getSystolic();
            }
            if (data.getDiastolic() > numDias) {
                numDias = data.getDiastolic();
            }
            if (data.getPulse() > numPulse) {
                numPulse = data.getPulse();
            }
        }
        systolicStats.getData().get(MAX).setYValue(numSys);
        diastolicStats.getData().get(MAX).setYValue(numDias);
        pulseStats.getData().get(MAX).setYValue(numPulse);

    }

    private void computeAvg() {
        double numSys = 1.;
        double numDias = 1.;
        double numPulse = 1.;

        if (sortedList.size() > 0) {
            numSys = 0.;
            numDias = 0.;
            numPulse = 0.;
            for (BPData data : sortedList) {
                numSys = numSys + data.getSystolic();
                numDias = numDias + data.getDiastolic();
                numPulse = numPulse + data.getPulse();
            }

            numSys = numSys / sortedList.size();
            numDias = numDias / sortedList.size();
            numPulse = numPulse / sortedList.size();
        }
        systolicStats.getData().get(AVG).setYValue(numSys);
        diastolicStats.getData().get(AVG).setYValue(numDias);
        pulseStats.getData().get(AVG).setYValue(numPulse);

    }

    private void computeMin() {
        Integer numSys = 1;
        Integer numDias = 1;
        Integer numPulse = 1;

        if (sortedList.size() > 0) {
            numSys = sortedList.get(0).getSystolic();
            numDias = sortedList.get(0).getDiastolic();
            numPulse = sortedList.get(0).getPulse();
            for (BPData data : sortedList) {
                if (data.getSystolic() < numSys) {
                    numSys = data.getSystolic();
                }
                if (data.getDiastolic() < numDias) {
                    numDias = data.getDiastolic();
                }
                if (data.getPulse() < numPulse) {
                    numPulse = data.getPulse();
                }
            }
        }
        systolicStats.getData().get(MIN).setYValue(numSys);
        diastolicStats.getData().get(MIN).setYValue(numDias);
        pulseStats.getData().get(MIN).setYValue(numPulse);

    }


}
