package com.asgteach.capitalpunishment.views;

import com.gluonhq.charm.glisten.application.MobileApplication;
import com.gluonhq.charm.glisten.control.AppBar;
import com.gluonhq.charm.glisten.mvc.View;
import com.gluonhq.charm.glisten.visual.MaterialDesignIcon;
import com.asgteach.capitalpunishment.CapitalPunishment;
import com.asgteach.capitalpunishment.model.Country;
import com.gluonhq.charm.glisten.animation.BounceInRightTransition;
import com.gluonhq.charm.glisten.layout.layer.FloatingActionButton;
import com.gluonhq.maps.MapLayer;
import com.gluonhq.maps.MapPoint;
import com.gluonhq.maps.demo.PoiLayer;
import javafx.fxml.FXML;
import javafx.geometry.Pos;
import javafx.scene.Node;
import javafx.scene.control.Label;
import javafx.scene.input.MouseEvent;
import javafx.scene.layout.Region;
import javafx.scene.layout.StackPane;
import javafx.scene.paint.Color;
import javafx.scene.shape.Circle;
import javafx.scene.text.TextAlignment;

public class MapPresenter {

    private com.gluonhq.maps.MapView mapView;
    private Country thePlace;
    private MapLayer mapLayer;

    @FXML
    private View view;

    public void initialize() {

        view.setShowTransitionFactory(BounceInRightTransition::new);

        view.showingProperty().addListener((obs, oldValue, newValue) -> {
            if (newValue) {
                AppBar appBar = 
                        MobileApplication.getInstance().getAppBar();
                appBar.setTitleText("Map");
                // new layer
                thePlace = CapitalPunishment.mapCenter;
                
                // remove the old layer
                if (mapLayer != null) {
                    mapView.removeLayer(mapLayer);
                }
                mapLayer = myDemoLayer(thePlace);
                mapView.addLayer(mapLayer);
                mapView.setZoom(3);
                mapView.setCenter(parsecoord(thePlace.getLatitude()),
                        parsecoord(thePlace.getLongitude()));
            } else {
                System.out.println("Map no longer showing");
            }

        });

        System.out.println("****Initializing MapView now");
        mapView = new com.gluonhq.maps.MapView();
        mapView.setZoom(3); // was 11
        view.setCenter(mapView);

        FloatingActionButton fb = new FloatingActionButton(
                        MaterialDesignIcon.HOME.text, e -> { 
                            MobileApplication.getInstance().switchView(
                                    CapitalPunishment.PRIMARY_VIEW);
        });
        view.getLayers().add(fb.getLayer());

    }

    private MapLayer myDemoLayer(Country country) {
        MapPoint target = new MapPoint(parsecoord(country.getLatitude()),
                parsecoord(country.getLongitude()));
        PoiLayer answer = new PoiLayer();
        System.out.println("creating MapLayer for " + 
                country.getCapital());
        Label label = new Label(country.getCapital() + ", " + 
                country.getName());
        label.setMinHeight(35);
        label.setMinWidth(Region.USE_PREF_SIZE);
        label.setTextAlignment(TextAlignment.CENTER);
        label.setStyle("-fx-padding: 5px; -fx-border-color:darkgray; "
                + "-fx-background-color: white; -fx-text-fill: darkgray;");

        StackPane sp = new StackPane(label);
        sp.setAlignment(Pos.CENTER);

        MapPoint infoPoint = new MapPoint(target.getLatitude() + 4, 
                target.getLongitude());
        Node icon1 = new Circle(7, Color.BLUE);
        icon1.setOnMouseClicked((MouseEvent me) -> {
            System.out.println("Mouse click");
            sp.setVisible(!sp.isVisible());
        });

        answer.addPoint(target, icon1);
        answer.addPoint(infoPoint, sp);
        return answer;
    }

    private double parsecoord(String geo) {
        double n;
        double m;
        try {
            n = Double.parseDouble(geo.substring(0, geo.length() - 1));
        } catch (NumberFormatException e) {
            return 0.0;     // Just return 0
        }
        char c = geo.charAt(geo.length() - 1);
        m = (n - Math.floor(n)) * 100;
        n = Math.floor(n) + (m * (1 / 60.0));
        if (c == 'S' || c == 'W') {
            n = n * (-1);
        }
        return n;
    }
}
